<?php

namespace Drupal\Tests\codemirror_editor\FunctionalJavascript;

/**
 * Tests the CodeMirror field formatter.
 *
 * @group codemirror_editor
 */
class FormatterTest extends TestBase {

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'field_ui',
  ];

  /**
   * Formatter settings.
   *
   * @var array
   */
  protected $formatterSettings = [];

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();
    $this->createNode([
      'type' => 'test',
      'field_code' => '<b class="bold">Example</b>',
    ]);
  }

  /**
   * Test callback.
   */
  public function testFormatter() {

    $permissions = [
      'administer node fields',
      'administer node display',
      'create test content',
      'edit any test content',
    ];
    $user = $this->drupalCreateUser($permissions);
    $this->drupalLogin($user);

    // Default formatter settings.
    $this->formatterSettings = [
      'mode' => 'text/html',
      'lineNumbers' => TRUE,
      'foldGutter' => FALSE,
    ];

    $this->drupalGet('node/1');
    $this->assertFormatter();

    $this->drupalGet('admin/structure/types/manage/test/display');

    $this->assertSettingsSummary();

    $this->click('//input[@name = "field_code_settings_edit"]');

    $this->assertSettingsForm();

    $this->formatterSettings = [
      'mode' => 'text/x-php',
      'lineNumbers' => FALSE,
      'foldGutter' => TRUE,
    ];

    $this->updateSettingField('mode');
    $this->updateSettingField('lineNumbers');
    $this->updateSettingField('foldGutter');

    $page = $this->getSession()->getPage();
    $page->pressButton('field_code_plugin_settings_update');

    $this->assertSession()
      ->waitForElementVisible('xpath', '//select[@name = "fields[field_code][type]"]');

    $page->pressButton('Save');
    $this->assertSession()->pageTextContains('Your settings have been saved.');
    $this->assertSettingsSummary();

    $this->drupalGet('node/1');
    $this->assertFormatter();
  }

  /**
   * Asserts formatter's output.
   */
  protected function assertFormatter() {
    $this->assertEditorOption('mode', $this->formatterSettings['mode']);
    $this->assertEditorOption('lineNumbers', $this->formatterSettings['lineNumbers']);
    $this->assertEditorOption('foldGutter', $this->formatterSettings['foldGutter']);
  }

  /**
   * Asserts formatter settings summary.
   */
  protected function assertSettingsSummary() {
    $expected_summary[] = 'Language mode: ' . $this->formatterSettings['mode'];
    $expected_summary[] = 'Line numbers: ' . ($this->formatterSettings['lineNumbers'] ? 'Yes' : 'No');
    $expected_summary[] = 'Fold gutter: ' . ($this->formatterSettings['foldGutter'] ? 'Yes' : 'No');

    $summary_xpath = '//tr[@id = "field-code"]//div[@class = "field-plugin-summary"]';
    $summary = $this->xpath($summary_xpath)[0]->getHtml();

    self::assertEquals(implode('<br>', $expected_summary), $summary);
  }

  /**
   * Asserts formatter settings form.
   */
  protected function assertSettingsForm() {
    $assert_session = $this->assertSession();

    $settings_wrapper = $assert_session
      ->waitForElementVisible('xpath', '//div[@data-drupal-selector = "edit-fields-field-code-settings-edit-form"]');

    $xpath = '//input[@name = "fields[field_code][settings_edit_form][settings][mode]" and @value = "%s"]';
    $xpath = sprintf($xpath, $this->formatterSettings['mode']);
    $assert_session->elementExists('xpath', $xpath, $settings_wrapper);

    $xpath = '//input[@name = "fields[field_code][settings_edit_form][settings][lineNumbers]" and %s]';
    $xpath = sprintf($xpath, $this->formatterSettings['lineNumbers'] ? '@checked = "checked"' : 'not(@checked)');
    $assert_session->elementExists('xpath', $xpath, $settings_wrapper);

    $xpath = '//input[@name = "fields[field_code][settings_edit_form][settings][foldGutter]" and %s]';
    $xpath = sprintf($xpath, $this->formatterSettings['foldGutter'] ? '@checked = "checked"' : 'not(@checked)');
    $assert_session->elementExists('xpath', $xpath, $settings_wrapper);
  }

  /**
   * Sets a value for a given settings field.
   */
  protected function updateSettingField($name) {
    $page = $this->getSession()->getPage();
    $field_name = "fields[field_code][settings_edit_form][settings][$name]";
    $value = $this->formatterSettings[$name];
    if (is_bool($value)) {
      $value ? $page->checkField($field_name) : $page->uncheckField($field_name);
    }
    else {
      $page->fillField($field_name, $value);
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function getWrapperSelector() {
    return '.field--name-field-code .cme-wrapper';
  }

}
